Package: ECoL
Type: Package
Version: 0.4.4
Date: 2026-05-19
Title: Complexity Measures for Supervised Problems
Description: Provides measures to characterize the complexity of classification 
    and regression problems based on aspects that quantify the linearity of the 
    data, the presence of informative feature, the sparsity and dimensionality 
    of the datasets. This package provides bug fixes, generalizations and 
    implementations of many state of the art measures. The measures are 
    described in the papers: Lorena et al. (2019) <doi:10.1145/3347711> and 
    Lorena et al. (2018) <doi:10.1007/s10994-017-5681-1>.
Authors@R: c(person(given = "Luis",
                   family = "Garcia",
                   role = c("aut", "cre"),
                   email = "luis.garcia@unb.br"),
            person(given = "Ana",
                   family = "Lorena",
                   role = c("aut", "ctb")),
            person(given = "Marcilio",
                   family = "Souto",
                   role = c("aut", "ctb")),
            person(given = c("Tin", "K."),
                   family = "Ho",
                   role = c("aut", "ctb")))
Maintainer: Luis Garcia <luis.garcia@unb.br>
URL: https://github.com/lpfgarcia/ECoL/
Imports: cluster, e1071, FNN, igraph, MASS
Suggests: testthat
License: MIT + file LICENSE
BugReports: https://github.com/lpfgarcia/ECoL/issues
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-19 17:51:46 UTC; lpfgarcia
Author: Luis Garcia [aut, cre],
  Ana Lorena [aut, ctb],
  Marcilio Souto [aut, ctb],
  Tin K. Ho [aut, ctb]
Repository: CRAN
Date/Publication: 2026-05-20 06:50:02 UTC
Built: R 4.5.2; ; 2026-05-20 07:40:31 UTC; unix
