# Generated by roxygen2: do not edit by hand

S3method(balance,default)
S3method(balance,formula)
S3method(complexity,default)
S3method(complexity,formula)
S3method(correlation,default)
S3method(correlation,formula)
S3method(dimensionality,default)
S3method(dimensionality,formula)
S3method(featurebased,default)
S3method(featurebased,formula)
S3method(linearity,default)
S3method(linearity,formula)
S3method(neighborhood,default)
S3method(neighborhood,formula)
S3method(network,default)
S3method(network,formula)
S3method(smoothness,default)
S3method(smoothness,formula)
export(balance)
export(complexity)
export(correlation)
export(dimensionality)
export(featurebased)
export(linearity)
export(neighborhood)
export(network)
export(smoothness)
export(summarization)
